﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace FLY.Integrated.UI.Client.UIModule
{
    /// <summary>
    /// UcIbcPanel.xaml 的交互逻辑
    /// </summary>
    public partial class UcIbcPanel : UserControl
    {
        FLY.Integrated.Client.IntegratedSystemServiceClient integratedSystemServiceClient;
        FLY.OBJComponents.Client.SetPLCUpdatePlan setPlan;

        public UcIbcPanel()
        {
            InitializeComponent();
        }

        public void Init()
        {
            integratedSystemServiceClient = TDGage.Current.integratedSystemServiceClient;
            this.DataContext = integratedSystemServiceClient.Ibc;

            //注册属性更新计划
            setPlan = new FLY.OBJComponents.Client.SetPLCUpdatePlan(
                integratedSystemServiceClient.PLCos,
                integratedSystemServiceClient.Ibc,
                new string[] {
                    "InletAirFreq",
                    "InletAirFreqSet",
                    "IsInletAirOn",

                    "OutletAirFreq",
                    "OutletAirFreqSet",
                    "IsOutletAirOn",
                    "IsOutletAirFreqChanged",

                    "ExCoolFreq",
                    "ExCoolFreqSet",
                    "IsExCoolOn",

                    "FilmWidth",
                    "FilmWidthSet",
                    "IsFilmWidthChanged",
                    "ErrorCorrection",

                    "IsIBCAuto"
            });


            integratedSystemServiceClient.Ibc.PropertyChanged += IbcData_PropertyChanged;
        }

        private async void IbcData_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (integratedSystemServiceClient.IsInPushValue)
                return;
            if (e.PropertyName == "FilmWidthSet")
            {
                await Task.Delay(1000);//1s后触发通知
                integratedSystemServiceClient.Ibc.IsFilmWidthChanged = true;
            }
            else if (e.PropertyName == "OutletAirFreqSet")
            {
                await Task.Delay(1000);//1s后触发通知
                integratedSystemServiceClient.Ibc.IsOutletAirFreqChanged = true;
            }
        }

        private void btnK123Click(object sender, RoutedEventArgs e)
        {
            WdK123 w = new WdK123();
            w.Init();
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            w.ShowDialog();
        }

        private void btnOutletAirPlusClick(object sender, RoutedEventArgs e)
        {
            if (integratedSystemServiceClient.Ibc.OutletAirFreqSet < 49.9)
                integratedSystemServiceClient.Ibc.OutletAirFreqSet += 0.1f;
        }
        private void btnOutletAirMinusClick(object sender, RoutedEventArgs e)
        {
            if (integratedSystemServiceClient.Ibc.OutletAirFreqSet > 0.1)
                integratedSystemServiceClient.Ibc.OutletAirFreqSet -= 0.1f;
        }
        private void btnInletAirPlusClick(object sender, RoutedEventArgs e)
        {
            if (integratedSystemServiceClient.Ibc.InletAirFreqSet < 49.9)
                integratedSystemServiceClient.Ibc.InletAirFreqSet += 0.1f;
        }

        private void btnInletAirMinusClick(object sender, RoutedEventArgs e)
        {
            if (integratedSystemServiceClient.Ibc.InletAirFreqSet > 0.1)
                integratedSystemServiceClient.Ibc.InletAirFreqSet -= 0.1f;
        }

        private void btnExCoolPlusClick(object sender, RoutedEventArgs e)
        {
            if (integratedSystemServiceClient.Ibc.ExCoolFreqSet < 49.9)
                integratedSystemServiceClient.Ibc.ExCoolFreqSet += 0.1f;
        }

        private void btnExCoolMinusClick(object sender, RoutedEventArgs e)
        {
            if (integratedSystemServiceClient.Ibc.ExCoolFreqSet > 0.1)
                integratedSystemServiceClient.Ibc.ExCoolFreqSet -= 0.1f;
        }
    }

    public class UIModule_IbcPanel : FLY.UI.Module.IUIModule
    {
        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title
        {
            get
            {
                return "综合.IBC调节";
            }
        }

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id)
        {
            UcIbcPanel graph = new UcIbcPanel();
            graph.Init();
            return graph;
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail(int id)
        {
            return new System.Windows.Controls.Grid();
        }


        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
        }
    }
}
