﻿using FLY.Integrated.Common;
using FLY.Integrated.IService;
using FLY.Integrated.OBJ_INTERFACE;
using FLY.Integrated.Server.Model;
using FLY.OBJComponents.Client;
using FLY.OBJComponents.IService;
using FObjBase;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Integrated.Client
{
    public class IntegratedSystemServiceClient : FObj, IIntegratedSystemService,INotifyPropertyChanged
    {
        public BufferServiceClient<Db_Width> CtrlList;
        #region IIntegratedSystemService
        /// <summary>
        /// 数据
        /// </summary>
        public IbcData Ibc { get; } = new IbcData();

        /// <summary>
        ///  收卷
        /// </summary>
        public WinderAccessory Accessory { get; } = new WinderAccessory();

        SyncPropServiceClient syncPropServiceClient;
        /// <summary>
        /// PLC代理系统
        /// </summary>
        public IPLCProxySystemService PLCos { get; private set; }

        #endregion

        public bool IsConnected { get; set; } = false;
    
        IFConn mConn;
        UInt32 mServerID;

        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// 当前 INotifyPropertyChanged 对象 引发了 PropertyChanged,  是由于 接收到数据导致的
        /// </summary>
        public bool IsInPushValue
        {
            get
            {
                return syncPropServiceClient.IsInPushValue;
            }
        }

        public IntegratedSystemServiceClient()
        {
            mServerID = OBJ_INTERFACE.OBJ_INTERFACE.IBC_OBJ_ID;

            syncPropServiceClient = new SyncPropServiceClient(
                mServerID + 1,
                new Dictionary<string, INotifyPropertyChanged> {
                    {"Ibc", Ibc },
                    {"Accessory", Accessory },
                });

            PLCos = new PLCProxySystemServiceClient(
                OBJ_INTERFACE.OBJ_INTERFACE.IBC_OBJ_PLCOS_ID,
                new Dictionary<string, INotifyPropertyChanged> {
                    {"Ibc", Ibc },
                    {"Accessory", Accessory }
                });


            CtrlList = new BufferServiceClient<Db_Width>(OBJ_INTERFACE.OBJ_INTERFACE.IBC_OBJ_CTRLLIST_ID);
        }
        public UInt32[] GetIDs()
        {
            List<UInt32> IDs = new List<uint>();
            IDs.Add(ID);
            IDs.Add(syncPropServiceClient.ID);
            IDs.AddRange(((PLCProxySystemServiceClient)PLCos).GetIDs());
            IDs.Add(CtrlList.ID);
            return IDs.ToArray();
        }
        public override void ConnectNotify(IFConn from)
        {
            mConn = from;

            IsConnected = from.IsConnected;
            if (from.IsConnected)
            {   
                CurrObjSys.SenseConfigEx(mConn, mServerID, ID,
                    0xffffffff, SENSE_CONFIG.ADD);
            }
        }

        

    }
}
