﻿using OfficeOpenXml;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Autogen_WS
{
    public class AnalysisAddr
    {
        /// <summary>
        /// 设备EP
        /// </summary>
        public DeviceAddrList deviceAddrs;

        /// <summary>
        /// 地址段映射
        /// </summary>
        DevMapperRuleTable devMapperRuleTable = new DevMapperRuleTable();

        /// <summary>
        /// 正则表达式
        /// </summary>
        string regexPattern;
        /// <summary>
        /// 设备名称Group序号
        /// </summary>
        int groupIdxOfDeviceName;
        /// <summary>
        /// 寄存器类型Group序号
        /// </summary>
        int groupIdxOfRegType;
        /// <summary>
        /// 地址Group序号
        /// </summary>
        int groupIdxOfAddr;


        public bool Load(string xlsxfile) 
        {
            int i_regexPattern = 1;
            int i_groupIdxOfDeviceName = 2;
            int i_groupIdxOfRegType = 3;
            int i_groupIdxOfAddr = 4;

            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["地址格式"])
                {
                    int iRow = 2;

                    try
                    {
                        regexPattern = sheet.Cells[iRow, i_regexPattern].Value.ToString();

                        string groupIdxOfDeviceName_str = sheet.Cells[iRow, i_groupIdxOfDeviceName].Value.ToString();
                        groupIdxOfDeviceName = int.Parse(groupIdxOfDeviceName_str);
                            
                        string groupIdxOfRegType_str = sheet.Cells[iRow, i_groupIdxOfRegType].Value.ToString();
                        groupIdxOfRegType = int.Parse(groupIdxOfRegType_str);


                        string groupIdxOfAddr_str = sheet.Cells[iRow, i_groupIdxOfAddr].Value.ToString();
                        groupIdxOfAddr = int.Parse(groupIdxOfAddr_str);
                    }
                    catch
                    {
                        return false;
                    }
                }
            }


            //加载转换表
            if (!devMapperRuleTable.Load(xlsxfile))
                return false;
            

            return true;
        }


        /// <summary>
        /// 分析 "XXX.D(6120)" 分拆为寄存器类型, 寄存器地址
        /// </summary>
        /// <param name="plc_addr">寄存器地址字符串</param>
        /// <param name="type">寄存器类型</param>
        /// <param name="addr">寄存器地址</param>
        /// <returns>是否成功</returns>
        public bool Resolve(string plc_addr, out int plc_index, out string type, out int addr)
        {

            type = null;
            addr = 0;
            plc_index = -1;
            //string plc_addr = "XXXXX.D(6120)";
            //Console.WriteLine("查找对象=" + plc_addr);


            Regex r = new Regex($@"{regexPattern}");//分析 D(6120)
            Match m = r.Match(plc_addr);
            if (!m.Success)
                return false;
            string plc_name = m.Groups[groupIdxOfDeviceName].Value;
            plc_index = deviceAddrs.deviceAddrInfos.FindIndex(info => info.deviceName == plc_name);
            if (plc_index == -1)
                return false;


            string plc_vt = m.Groups[groupIdxOfRegType].Value;

            //查表
            var cells = from _cell in devMapperRuleTable.Tables where _cell.PLCVarType == plc_vt select _cell;
            if (cells.Count() == 0)
                return false;
            var rule = cells.First();

            type = rule.VarType;
            //特别的， M0~...   D0~.... 是10进制
            //但 X0~X7 ,X10~X17,X70~X77,X100~X107, Y0~Y7 ,Y10~Y17,  是8进制

            string addr_str = m.Groups[groupIdxOfAddr].Value;

            int plc_a = Convert.ToInt32(addr_str, rule.FromBase);

            addr = plc_a + rule.Addr1st;

            return true;

        }
    }
    
}
