﻿using OfficeOpenXml;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Autogen_WS
{
    /// <summary>
    /// 类表
    /// </summary>
    public class ClassList
    {
        public List<ClassInfo> classInfos = new List<ClassInfo>();


        /// <summary>
        /// 读取设备转换表
        /// </summary>
        /// <returns></returns>
        public bool Load(string xlsxfile)
        {
            classInfos.Clear();
            /// <summary>
            /// 类名
            /// </summary>
            int i_className = 1;
            /// <summary>
            /// 表名
            /// </summary>
            int i_tableName = 2;
            /// <summary>
            /// 单例模式
            /// </summary>
            int i_isSingleton = 3;
            /// <summary>
            /// 实例名
            /// </summary>
            int i_objectName = 4;


            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["类表"])
                {
                    int iRow = 2;
                    int i = iRow - 1;
                    while (true)
                    {
                        i++;
                        ClassInfo classInfo = new ClassInfo();
                        try
                        {
                            classInfo.className = sheet.Cells[i, i_className].Value.ToString();
                            classInfo.tableName = sheet.Cells[i, i_tableName].Value.ToString();
                            
                            string isSingleton_str = sheet.Cells[iRow, i_isSingleton].Value.ToString();
                            if (!string.IsNullOrEmpty(isSingleton_str))
                            {
                                if (Regex.IsMatch(isSingleton_str, "Y"))
                                {
                                    classInfo.isSingleton = true;
                                }
                            }
                            classInfo.objectName = sheet.Cells[i, i_objectName].Value.ToString();
                        }
                        catch
                        {
                            //完成，没有数据了
                            break;
                        }

                        if (string.IsNullOrEmpty(classInfo.className)) break;

                        if (string.IsNullOrEmpty(classInfo.tableName)) return false;

                        if (string.IsNullOrEmpty(classInfo.objectName)) return false;

                        classInfos.Add(classInfo);

                    }
                }
            }
            if (classInfos.Count == 0)
                return false;
            return true;
        }

    }
    public class ClassInfo 
    {
        /// <summary>
        /// 类名
        /// </summary>
        public string className;
        /// <summary>
        /// 表名
        /// </summary>
        public string tableName;
        /// <summary>
        /// 单例模式
        /// </summary>
        public bool isSingleton;
        /// <summary>
        /// 实例名
        /// </summary>
        public string objectName;
    }
}
