﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using OfficeOpenXml;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Autogen_WS
{
    public class DataInfos:INotifyPropertyChanged
    {
        /// <summary>
        /// 输出的C#对象属性表
        /// </summary>
        List<WS_ItemInfo> infos = new List<WS_ItemInfo>();

        /// <summary>
        /// 设备EP
        /// </summary>
        DeviceAddrList deviceAddrs = new DeviceAddrList();

        /// <summary>
        /// 类名
        /// </summary>
        ClassList classList = new ClassList();

        /// <summary>
        /// 地址分析器
        /// </summary>
        AnalysisAddr analysisAddr = new AnalysisAddr();

        /// <summary>
        /// 地址段映射
        /// </summary>
        DevMapperRuleTable devMapperRuleTable = new DevMapperRuleTable();

        /// <summary>
        /// 主界面的dispatcher
        /// </summary>
        public Action<string> ErrorMsgbox;

        #region 参数
        /// <summary>
        /// 输出的xml文件
        /// </summary>
        string XlsxFile;
        #endregion

        #region 属性
        /// <summary>
        /// 进度
        /// </summary>
        public double Progress { get; set; }


        #endregion
        public DataInfos()
        {

        }


        public bool LoadXlsxFile(string xlsxFile) 
        {
            XlsxFile = xlsxFile;
            //加载类表
            if (!classList.Load(XlsxFile)) {
                ErrorMsgbox?.Invoke("加载类表失败");
                return false;
            }

            //加载地址
            if (!deviceAddrs.Load(XlsxFile))
            {
                ErrorMsgbox?.Invoke("加载地址失败");
                return false;
            }

            analysisAddr.deviceAddrs = deviceAddrs;
            //加载转换表
            if (!analysisAddr.Load(XlsxFile))
            {
                ErrorMsgbox?.Invoke("加载转换器失败");
                return false;
            }

            //加载 & 转换地址
            return LoadItemInfo(XlsxFile);
        }

        /// <summary>
        /// 加载 HMI-地址表.xlsm
        /// </summary>
        /// <param name="xlsxfile"></param>
        /// <returns></returns>
        bool LoadItemInfo(string xlsxfile) 
        {
            infos.Clear();
            Progress = 0;
            for (int i = 0; i < classList.classInfos.Count; i++)
            {
                if (!LoadItemInfo_XXX(xlsxfile, classList.classInfos[i]))
                    return false;
            }
            return true;
        }
        bool LoadItemInfo_XXX(string xlsxfile, ClassInfo classInfo)
        {
            int i_propDescription = 2;//描述
            int i_propName = 3;//属性名
            int i_typeName = 4;//类型
            int i_scale = 5;//放大
            int i_canWrite = 6;//可写
            int i_isError = 7;//报警
            int i_trigger = 8;//触发
            int i_plcAddr = 9;//PLC寄存器

            int item_cnt = 0;//非单例，多个实例

            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets[classInfo.tableName])
                {
                    if (!classInfo.isSingleton)
                    {
                        int i = i_plcAddr;

                        Regex r_header_addr = new Regex(@"HMI寄存器");

                        while (true)
                        {
                            string str;
                            try
                            {
                                str = sheet.Cells[1, i].Value.ToString();
                            }
                            catch
                            {
                                break;
                            }
                            if (str == null) break;
                            if (str == "") break;

                            if (r_header_addr.IsMatch(str))
                            {
                                item_cnt++;
                            }
                            i++;
                            continue;
                        }
                        if (item_cnt < 1)
                        {
                            ErrorMsgbox?.Invoke("\"HMI寄存器\" 找不到");
                            return false;
                        }
                    }

                    int iRow = 2;
                    while (true)
                    {
                        WS_ItemInfo ii = new WS_ItemInfo();

                        //描述
                        try
                        {
                            ii.propDescription = sheet.Cells[iRow, i_propDescription].Value.ToString();
                        }
                        catch
                        {
                            break;//没有描述，肯定就没有数据了
                        }


                        //名称
                        try
                        {
                            ii.propName = sheet.Cells[iRow, i_propName].Value.ToString();
                        }
                        catch
                        {
                            //没有名称异常
                            ErrorMsgbox?.Invoke("没有名称异常 描述=" + ii.propDescription);
                            return false;
                        }


                        //类型
                        try
                        {
                            ii.typeName = sheet.Cells[iRow, i_typeName].Value.ToString();
                        }
                        catch
                        {
                            //没有类型异常
                            ErrorMsgbox?.Invoke("没有类型异常 描述=" + ii.propDescription);
                            return false;
                        }

                        //放大
                        try
                        {
                            string scale_str = sheet.Cells[iRow, i_scale].Value.ToString();
                            if (!string.IsNullOrEmpty(scale_str))
                            {
                                double d;
                                if (double.TryParse(scale_str, out d))
                                {
                                    ii.scale = d;
                                }
                            }
                        }
                        catch
                        {

                        }

                        //报警
                        try
                        {
                            string isError_str = sheet.Cells[iRow, i_isError].Value.ToString();
                            if (!string.IsNullOrEmpty(isError_str))
                            {
                                if (Regex.IsMatch(isError_str, "Y"))
                                {
                                    ii.isError = true;
                                }
                            }
                        }
                        catch
                        {

                        }

                        //触发
                        try
                        {
                            string trigger_str = sheet.Cells[iRow, i_trigger].Value.ToString();
                            if (!string.IsNullOrEmpty(trigger_str))
                            {
                                if (Regex.IsMatch(trigger_str, "OFF"))
                                {
                                    ii.trigger = false;
                                }
                            }
                        }
                        catch
                        {

                        }


                        //把多余的空格删除
                        ii.propName = ii.propName.Trim();
                        ii.typeName = ii.typeName.Trim();
                        ii.propDescription = ii.propDescription.Trim();


                        if (classInfo.isSingleton)
                        {
                            ii.objectName = classInfo.objectName;
                            //propertyname 不能有重复
                            if (infos.Any((info) => { return (info.propName == ii.propName) && (info.objectName == ii.objectName); }))
                            {
                                ErrorMsgbox?.Invoke($"相同的属性名:{ii.propName} 对象:{ii.objectName} 描述:{ii.propDescription}");
                                return false;
                            }

                            string plc_addr;
                            try
                            {
                                plc_addr = sheet.Cells[iRow, i_plcAddr].Value.ToString();
                            }
                            catch
                            {
                                //空的
                                ErrorMsgbox?.Invoke("没有PLC地址 描述=" + ii.propDescription);
                                return false;
                            }

                            if (!analysisAddr.Resolve(plc_addr, out ii.plcIndex, out ii.regType, out ii.mbAddr))
                            {
                                ErrorMsgbox?.Invoke($"{plc_addr} 不能转换为modbus地址 描述:{ii.propDescription}");
                                return false;
                            }
                            infos.Add(ii);
                            ii = ii.Clone();
                        }
                        else 
                        {
                            for (int i = 0; i < item_cnt; i++) 
                            {
                                ii.objectName = $"{classInfo.objectName}[{i}]";

                                if (infos.Any((info) => { return (info.propName == ii.propName) && (info.objectName == ii.objectName); }))
                                {
                                    ErrorMsgbox?.Invoke($"相同的属性名:{ii.propName} 对象:{ii.objectName} 描述:{ii.propDescription}");
                                    return false;
                                }

                                string plc_addr;
                                try
                                {
                                    plc_addr = sheet.Cells[iRow, i_plcAddr + i].Value.ToString();
                                }
                                catch
                                {
                                    //空的
                                    continue;
                                }

                                if (!analysisAddr.Resolve(plc_addr, out ii.plcIndex, out ii.regType, out ii.mbAddr))
                                {
                                    ErrorMsgbox?.Invoke($"{plc_addr} 不能转换为modbus地址 描述:{ii.propDescription}");
                                    return false;
                                }

                                infos.Add(ii);
                                ii = ii.Clone();
                            }
                        }



                        iRow++;
                        Progress += 1;
                        if (Progress >= 100)
                            Progress = 50;
                    }
                    Progress = 100;
                    return true;
                }
            }
        }
        void Gen_CS_Core(StreamWriter sw, Dictionary<string, WS_ItemInfo> d)
        {
            sw.WriteLine("        #region 辅助代码生成");
            foreach (KeyValuePair<string, WS_ItemInfo> sii in d)
            {
                WS_ItemInfo info = sii.Value;

                string pre = "        ";
                sw.WriteLine($"{pre}/// <summary>");
                if (info.isError)
                {
                    if (info.trigger)
                        sw.WriteLine($"{pre}/// {info.propDescription} (ON)");
                    else
                        sw.WriteLine($"{pre}/// {info.propDescription} (OFF)");
                }
                else
                {
                    sw.WriteLine($"{pre}/// {info.propDescription}");
                }
                sw.WriteLine($"{pre}/// </summary>");
                sw.WriteLine($"{pre}[Description(\"{ info.propDescription}\")]");
                if (info.isError)
                {
                    if (info.trigger)
                    {
                        sw.WriteLine($"{pre}[IsError()]");
                    }
                    else
                    {
                        sw.WriteLine($"{pre}[IsError(offIsError:true)]");
                    }
                }
                sw.WriteLine($"{pre}public {info.GetPropertyType()} {info.propName}{{ get; set; }}");
                sw.WriteLine();
            }
            sw.WriteLine("        #endregion");
        }

        void Gen_CS_XXX(StreamWriter sw, ClassInfo classInfo)
        {

            Dictionary<string, WS_ItemInfo> d = new Dictionary<string, WS_ItemInfo>();

            if (classInfo.isSingleton)
            {
                foreach (WS_ItemInfo ii in infos)
                {
                    if (ii.objectName != classInfo.objectName) continue;
                    if (d.ContainsKey(ii.propName)) continue;
                    d.Add(ii.propName, ii);
                }
            }
            else 
            {
                foreach (WS_ItemInfo ii in infos)
                {
                    if (!ii.objectName.StartsWith(classInfo.objectName))
                        continue;
                    if (!Regex.IsMatch(ii.objectName, @"\[\d+\]"))
                        continue;

                    if (d.ContainsKey(ii.propName))
                        continue;
                    d.Add(ii.propName, ii);
                }
            }

            Gen_CS_Core(sw, d);

        }

        void Gen_CS(ClassInfo classInfo)
        {
            //找到 #region 辅助代码生成 与最接近的 #endregion

            int state = 0;
            int region_cnt = 0;

            string cs_new_path = Path.Combine(Path.GetDirectoryName(XlsxFile), "Generated", classInfo.className + ".cs");
            string csharppath = Path.Combine(Path.GetDirectoryName(XlsxFile), classInfo.className + ".cs");

            if (!Directory.Exists(Path.GetDirectoryName(cs_new_path)))
                Directory.CreateDirectory(Path.GetDirectoryName(cs_new_path));

            StreamWriter sw = new StreamWriter(cs_new_path, false, Encoding.UTF8);

            if (File.Exists(csharppath))
            {
                StreamReader sr = new StreamReader(csharppath);

                //把整个文件读到 sb中
                while (!sr.EndOfStream)
                {
                    string line = sr.ReadLine();
                    switch (state)
                    {
                        case 0:
                            if (line.Contains("#region 辅助代码生成"))
                            {
                                //找到了
                                state = 1;
                                region_cnt = 1;
                            }
                            else
                            {
                                sw.WriteLine(line);
                            }
                            break;
                        case 1://跳过中间的
                            if (line.Contains("#region"))
                            {
                                region_cnt++;
                            }
                            else if (line.Contains("#endregion"))
                            {
                                region_cnt--;
                                if (region_cnt == 0)
                                {
                                    //插入 自动代码
                                    Gen_CS_XXX(sw, classInfo);

                                    //下面都是尾
                                    state = 2;
                                }
                            }
                            break;

                        case 2://记录尾巴
                            sw.WriteLine(line);
                            break;
                    }
                }

                sr.Close();
                sw.Flush();
                sw.Close();
            }
            else
            {
                sw.WriteLine($"using System.ComponentModel;");
                sw.WriteLine();
                sw.WriteLine($"namespace XXX.Common");
                sw.WriteLine($"{{");
                sw.WriteLine($"    public class {classInfo.className} : INotifyPropertyChanged");
                sw.WriteLine($"    {{");
                sw.WriteLine($"        public {classInfo.className}()");
                sw.WriteLine($"        {{");
                sw.WriteLine();
                sw.WriteLine($"        }}");
                Gen_CS_XXX(sw,classInfo);
                sw.WriteLine($"        public event PropertyChangedEventHandler PropertyChanged;");
                sw.WriteLine($"    }}");
                sw.WriteLine($"}}");
                sw.Flush();
                sw.Close();
            }
        }

        /// <summary>
        /// 生成 cs文件
        /// </summary>
        /// <returns></returns>
        public bool Gen_CS()
        {
            foreach (var classInfo in classList.classInfos) 
            {
                Gen_CS(classInfo);
            }
            return true;
        }
        public bool Gen_WS_XML()
        {
            PLCGroup plcgroup = new PLCGroup();

            for (int i=0;i< deviceAddrs.deviceAddrInfos.Count(); i++)
            {
                string ep_str = deviceAddrs.deviceAddrInfos[i].ep;
                plcgroup.Devices.Add(new PLCGroup.PLCDevice() { EP = ep_str });
            }
            foreach (WS_ItemInfo ii in infos)
            {
                plcgroup.Variables.Add(new PLCGroup.PLCVariable()
                {
                    DeviceIndex = ii.plcIndex,
                    Mode = ii.regType,
                    Addr = ii.mbAddr,
                    Type = ii.typeName,
                    Scale = ii.scale,
                    OwnerName = ii.objectName,
                    PropertyName = ii.propName
                });
            }

            string ws_path = Path.Combine(Path.GetDirectoryName(XlsxFile),"Generated","WS.json");
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(plcgroup, Newtonsoft.Json.Formatting.Indented);
            File.WriteAllText(ws_path, json);
            return true;
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }


}
